Exception: Ask::Graph::WorkflowTimeout
- Inherits:
-
StandardError
- Object
- StandardError
- Ask::Graph::WorkflowTimeout
- Defined in:
- lib/ask/graph/runner.rb
Overview
Error raised when the entire workflow exceeds its total runtime cap.
Error raised when the entire workflow exceeds its total runtime cap.